@@ -1,5 +1,5 @@
 # Device::Gsm - a Perl class to interface GSM devices as AT modems
-# Copyright (C) 2002-2004 Cosimo Streppone, cosimo@cpan.org
+# Copyright (C) 2002-2008 Cosimo Streppone, cosimo@cpan.org
 #
 # This program is free software; you can redistribute it and/or modify
 # it only under the terms of Perl itself.
@@ -9,17 +9,16 @@
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # Perl licensing terms for more details.
 #
-# Additionally, this is now ALPHA software, still needs extensive
-# testing and support for custom GSM commands, so use it at your own risk,
-# and without ANY warranty! Have fun.
+# Commercial support is available. Write me if you are
+# interested in new features or software support.
 #
-# $Id: Gsm.pm,v 1.36 2004/09/15 21:39:21 cosimo Exp $
+# $Id$
 
 package Device::Gsm;
-$Device::Gsm::VERSION = sprintf "%d.%02d", q$Revision: 1.36 $ =~ /(\d+)\.(\d+)/;
+$Device::Gsm::VERSION = '1.52';
 
 use strict;
-use Device::Modem;
+use Device::Modem 1.47;
 use Device::Gsm::Sms;
 use Device::Gsm::Pdu;
 use Device::Gsm::Charset;
@@ -95,17 +94,17 @@ sub datetime {
 
 			# Set time of phone
 			$self->atsend( qq{AT+CCLK="$datetime"} . Device::Modem::CR );
-			$ok = $self->parse_answer();
+			$ok = $self->parse_answer($Device::Modem::STD_RESPONSE);
 
 			$self->log->write('info', "write datetime ($datetime) to phone => (".($ok?'OK':'FAILED').")");
 
 		} else {
 
 			$self->atsend( 'AT+CCLK?' . Device::Modem::CR );
-			($ok, $datetime) = $self->parse_answer();
+			($ok, $datetime) = $self->parse_answer($Device::Modem::STD_RESPONSE);
 		
 			#warn('datetime='.$datetime);
-			if( $ok && $datetime =~ m|\+CCLK: "(\d\d)/(\d\d)/(\d\d)\,(\d\d):(\d\d):(\d\d)"| ) {
+			if( $ok && $datetime =~ m|\+CCLK:\s*"?(\d\d)/(\d\d)/(\d\d)\,(\d\d):(\d\d):(\d\d)"?| ) {
 				$datetime = "$1/$2/$3 $4:$5:$6";
 				$self->log->write('info', "read datetime from phone ($datetime)");
 			} else {
@@ -125,28 +124,65 @@ sub datetime {
 # Delete a message from sim card
 #
 sub delete_sms {
-    my $self      = $_[0];
-    my $msg_index = $_[1];
+    my $self      = shift;
+    my $msg_index = shift;
+    my $storage   = shift;
     my $ok;
 
-    if( ! defined $msg_index || ! $msg_index ) {
+    if( ! defined $msg_index || $msg_index eq '' ) {
         $self->log->write('warn', 'undefined message number. cannot delete sms message');
         return 0;
     }
-       
-    $self->atsend( qq{AT+CMGD=$msg_index} . Device::Modem::CR );
-    $self->wait(500);
 
-    my $ans = $self->parse_answer();
+    # Set default SMS storage if supported
+    $self->storage($storage);
+
+    $self->atsend( qq{AT+CMGD=$msg_index} . Device::Modem::CR );
+
+    my $ans = $self->parse_answer($Device::Modem::STD_RESPONSE);
     if( index($ans, 'OK') > -1 || $ans =~ /\+CMGD/ ) {
         $ok = 1;
     }
 
-    $self->log->write('info', "deleting sms n.$msg_index from sim card (result: `$ans') => " . ($ok ? 'ok' : '*FAILED*') );
+    $self->log->write('info', "deleting sms n.$msg_index from storage ".($storage || "default")." (result: `$ans') => " . ($ok ? 'ok' : '*FAILED*') );
 
     return $ok;
 }
 
+#
+# Call forwarding
+#
+sub forward {
+    my ($self, $reason, $mode, $number) = @_;
+
+    $reason = lc $reason || 'unconditional';
+    $mode   = lc $mode   || 'register';
+    $number ||= '';
+
+    my %reasons = (
+        'unconditional' => 0,
+        'busy'          => 1,
+        'no reply'      => 2,
+        'unreachable'   => 3
+    );
+
+    my %modes = (
+        'disable'  => 0,
+        'enable'   => 1,
+        'query'    => 2,
+        'register' => 3,
+        'erase'    => 4
+    );
+
+    my $reasoncode = $reasons{$reason};
+    my $modecode = $modes{$mode};
+
+    $self->log->write('info', qq{setting $reason call forwarding to [$number]});
+    $self->atsend( qq{AT+CCFC=$reasoncode,$modecode,"$number"} . Device::Modem::CR );
+
+    return $self->parse_answer($Device::Modem::STD_RESPONSE, 15000);
+}
+
 #
 # Hangup and terminate active call(s)
 # this overrides the `Device::Modem::hangup()' method
@@ -157,28 +193,40 @@ sub hangup {
 	$self->attention();
 	$self->atsend( 'AT+CHUP' . Device::Modem::CR );
 	$self->flag('OFFHOOK', 0);
-	$self->answer();
+	$self->answer(undef, 5000);
 }
 
 #
 # Who is the manufacturer of this device?
 #
-sub manufacturer() {
-	my $self = shift;
-	my($ok, $man);
+sub manufacturer
+{
+    my $self = shift;
+    my($ok, $man);
 
-	# Test if manufacturer code command is supported
-	if( $self->test_command('+CGMI') ) {
-
-		$self->atsend( 'AT+CGMI' . Device::Modem::CR );
-		($ok, $man) = $self->parse_answer();
-
-		$self->log->write('info', 'manufacturer of this device appears to be ['.$man.']');
+	# We can't test for command support, because some phones, mainly Motorola
+	# will spit out an error, instead of telling if CGMI is supported.
+	$self->atsend( 'AT+CGMI' . Device::Modem::CR );
+	($ok, $man) = $self->parse_answer($Device::Modem::STD_RESPONSE);
 
+	if($ok ne 'OK')
+	{
+		$self->log->write('warn', 'manufacturer command ended with error [' . $ok . $man . ']');
+		return undef;
 	}
 
-	return $man || $ok;
+	# Again, seems that Motorola phones will re-echo
+	# the CGMI command header, instead of giving us the
+	# manufacturer info we want. Thanks to Niolay Shaplov
+	# for reporting (RT #31540)
+	if($man =~ /\+CGMI:\ \"(.*)\"/s)
+	{
+		$man = $1;
+	}
 
+	$self->log->write('info', 'manufacturer of this device appears to be ['.$man.']');
+
+	return $man || $ok;
 }
 
 #
@@ -197,9 +245,8 @@ sub mode {
         $self->{'_mode'} = $mode ? 'text' : 'pdu';
         $self->log->write('info', 'setting mode to ['.$self->{'_mode'}.']');
         $self->atsend( qq{AT+CMGF=$mode} . Device::Modem::CR );
-        $self->wait(100);
 
-        return $self->parse_answer();
+        return $self->parse_answer($Device::Modem::STD_RESPONSE);
     }
 
     return($self->{'_mode'}||'');
@@ -209,7 +256,7 @@ sub mode {
 #
 # What is the model of this device?
 #
-sub model() {
+sub model {
 	my $self = shift;
 	my($code, $model);
 
@@ -217,7 +264,7 @@ sub model() {
 	if( $self->test_command('+CGMM') ) {
 
 		$self->atsend( 'AT+CGMM' . Device::Modem::CR );
-		($code, $model) = $self->parse_answer();
+		($code, $model) = $self->parse_answer($Device::Modem::STD_RESPONSE);
 
 		$self->log->write('info', 'model of this device is ['.($model||'').']');
 
@@ -229,7 +276,7 @@ sub model() {
 #
 # Get handphone serial number (IMEI number)
 #
-sub imei() {
+sub imei {
 	my $self = shift;
 	my($code,$imei);
 
@@ -237,7 +284,7 @@ sub imei() {
 	if( $self->test_command('+CGSN') ) {
 
 		$self->atsend( 'AT+CGSN' . Device::Modem::CR );
-		($code, $imei) = $self->parse_answer();
+		($code, $imei) = $self->parse_answer($Device::Modem::STD_RESPONSE);
 
 		$self->log->write('info', 'IMEI code is ['.$imei.']');
 
@@ -252,18 +299,31 @@ sub imei() {
 #
 # Get mobile phone signal quality (expressed in dBm)
 #
-sub signal_quality() {
+sub signal_quality {
 	my $self = shift;
 	# Error code, dBm (signal power), bit error rate
-	my($code, $dBm, $ber);
+	my($code, @dBm, $dBm, $ber);
 
 	# Test if signal quality command is implemented
 	if( $self->test_command('+CSQ') ) {
 
 		$self->atsend( 'AT+CSQ' . Device::Modem::CR );
-		($code, $dBm) = $self->parse_answer();
+		($code, @dBm) = $self->parse_answer($Device::Modem::STD_RESPONSE, 15000);
 
-		if( $dBm =~ /\+CSQ: (\d+),(\d+)/ ) {
+		# Vodafone data cards send out response to commands with
+		# many empty lines in between, so +CSQ response is not the very
+		# first line of answer. 
+		for(@dBm)
+		{
+			if( /\+CSQ:/ )
+			{
+				$dBm = $_;
+				last;
+			}
+		}
+
+		# Some gsm software send CSQ command result as "+CSQ: xx,yy"
+		if( $dBm =~ /\+CSQ:\s*(\d+),(\d+)/ ) {
 
 			($dBm, $ber) = ($1, $2);
 
@@ -276,6 +336,13 @@ sub signal_quality() {
 
 			$self->log->write('info', 'signal dBm power is ['.$dBm.'], bit error rate ['.$ber.']');
 
+		# Other versions put out "+CSQ: xx" only...
+		} elsif( $dBm =~ /\+CSQ:\s*(\d+)/ ) {
+
+			$dBm = $1;
+			
+			$self->log->write('info', 'signal is ['.$dBm.'] "bars"');
+
 		} else {
 
 			$self->log->write('warn', 'cannot obtain signal dBm power');
@@ -296,7 +363,7 @@ sub signal_quality() {
 #
 # Get the GSM software version on this device
 #
-sub software_version() {
+sub software_version {
 	my $self = shift;
 	my($code, $ver);
 
@@ -304,7 +371,7 @@ sub software_version() {
 	if( $self->test_command('+CGMR') ) {
 
 		$self->atsend( 'AT+CGMR' . Device::Modem::CR );
-		($code, $ver) = $self->parse_answer();
+		($code, $ver) = $self->parse_answer($Device::Modem::STD_RESPONSE);
 
 		$self->log->write('info', 'GSM version is ['.$ver.']');
 
@@ -330,7 +397,7 @@ sub test_command {
 	$self->atsend( "AT$command=?" . Device::Modem::CR );
 
 	# If answer is ok, command is supported
-	my $ok = ($self->answer() || '') =~ /OK/o;
+	my $ok = ($self->answer($Device::Modem::STD_RESPONSE) || '') =~ /OK/o;
 	$self->log->write('info', 'command ['.$command.'] is '.($ok ? '' : 'not ').'supported');
 
 	$ok;
@@ -339,10 +406,15 @@ sub test_command {
 #
 # Read all messages on SIM card (XXX must be registered on network)
 #
-sub messages {
-	my $self = shift;
+sub messages
+{
+    my($self, $storage) = @_;
     my @messages;
-    $self->log->write('info', 'reading messages on SIM card');
+
+    # By default (old behaviour) messages are read from sim card
+    $storage ||= 'SM';
+
+    $self->log->write('info', 'Reading messages on ' . ($storage eq 'SM' ? 'Sim card' : 'phone memory'));
 
     # Register on network (give your PIN number for this!)
     #return undef unless $self->register();
@@ -351,15 +423,40 @@ sub messages {
     #
     # Read messages (TODO need to check if device supports CMGL with `stat'=4)
     #
-    if( $self->mode() eq 'text' ) {
+    if( $self->mode() eq 'text' )
+    {
         @messages = $self->_read_messages_text();
-    } else {
-	    @messages = $self->_read_messages_pdu();
+    }
+    else
+    {
+        # Set default storage if supported
+        $self->storage($storage);
+
+        push @messages, $self->_read_messages_pdu();
     }
 
     return @messages;
 }
 
+sub storage
+{
+    my $self = shift;
+    my $ok = 0;
+
+    # Set default SMS storage if supported by phone
+    if( @_ && (my $storage = uc $_[0]) )
+    {
+        return unless $self->test_command('+CPMS');
+        $self->atsend(qq{AT+CPMS="$storage"} . Device::Modem::CR);
+
+        # Read and discard the answer
+        $self->answer($Device::Modem::STD_RESPONSE, 5000);
+        $self->{_storage} = $storage;
+    }
+
+    return $self->{_storage};
+}
+
 #
 # Register to GSM service provider network
 #
@@ -381,9 +478,10 @@ sub register {
 	$me->atsend( 'AT+CPIN?' . Device::Modem::CR );
 
 	# Get answer
-	my $cReply = $me->answer();
+	my $cReply = $me->answer($Device::Modem::STD_RESPONSE, 10000);
 
-	if( $cReply =~ /READY/ ) {
+	if( $cReply =~ /(READY|SIM PIN2)/ ) {
+		# Iridium satellite phones rest saying "SIM PIN2" when they are registered...
 
 		$me->log->write( 'info', 'Already registered on network. Ready to send.' );
 		$lOk = 1;
@@ -395,7 +493,7 @@ sub register {
 		$me->atsend( qq[AT+CPIN="$$me{'pin'}"] . Device::Modem::CR );
 
 		# Get reply
-		$cReply = $me->answer();
+		$cReply = $me->answer($Device::Modem::STD_RESPONSE, 10000);
 
 		# Test reply
 		if( $cReply !~ /ERROR/ ) {
@@ -469,18 +567,27 @@ sub send_sms {
 
 #
 #
-# read sim messages in pdu mode
+# read messages in pdu mode
 #
 #
-sub _read_messages_pdu {
-    my $self = $_[0];
+sub _read_messages_pdu
+{
+    my $self = shift;
 
     $self->mode('pdu');
+
     $self->atsend( q{AT+CMGL=4} . Device::Modem::CR);
-	my($messages) = $self->answer();
+	my($messages) = $self->answer($Device::Modem::STD_RESPONSE, 5000);
+
+	# Catch the case that the msgs are returned with gaps between them
+	while (my $more = $self->answer($Device::Modem::STD_RESPONSE, 200)) {
+        #-- $self->answer will chomp trailing newline, add it back
+		$messages .= "\n";
+		$messages .= $more;
+	}
 
 	# Ok, messages read, now convert from PDU and store in object
-	$self->log->write('debug', 'messages='.$messages );
+	$self->log->write('debug', 'Messages='.$messages );
 
 	my @data = split /[\r\n]+/m, $messages;
 
@@ -494,18 +601,23 @@ sub _read_messages_pdu {
 	my @message = ();
 	my $current;
 
+    # Current sms storage memory (ME or SM)
+    my $storage = $self->storage();
+
 	#
 	# Parse received data (result of +CMGL command)
 	#
 	while( @data ) {
 
 		$self->log->write('debug', 'data[] = ', $data[0] );
-
+		my $header=shift @data;
+		my $pdu   =shift @data;
 		# Instance new message object
 		my $msg = new Device::Gsm::Sms(
-			header => shift @data,
-			pdu    => shift @data,
+			header => $header,
+			pdu    => $pdu,
             # XXX mode   => $self->mode(),
+            storage => $storage,
             parent => $self                  # Ref to parent Device::Gsm class
 		);
 
@@ -513,7 +625,7 @@ sub _read_messages_pdu {
 		if( ref $msg ) {
 			push @message, $msg;
 		} else {
-			$self->log->write('info', 'could not instance message!');
+			$self->log->write('info', "could not instance message $header $pdu!");
 		}
 
 	}
@@ -553,14 +665,14 @@ sub _send_sms_text {
 
 	# Get reply and check for errors
 	$cReply = $me->answer('\+CMGS', 2000);
-	if( $cReply =~ /ERROR/i ) {
-		$me->log->write( 'warning', "ERROR in sending SMS" );
-	} else {
+	if( $cReply =~ /OK$/i ) {
 		$me->log->write( 'info', "Sent SMS (text mode) to $num!" );
 		$lOk = 1;
+	} else {
+		$me->log->write( 'warning', "ERROR in sending SMS" );
 	}
 
-	$lOk
+	return $lOk;
 }
 
 
@@ -576,6 +688,9 @@ sub _send_sms_pdu {
 
 	return 0 unless $num and $text;
 
+	$me->atsend(  q[ATE1] . Device::Modem::CR );
+	$me->answer($Device::Modem::STD_RESPONSE);
+
 	# Select class of sms (normal or *flash sms*)
 	my $class = $opt{'class'} || 'normal';
 	$class = $class eq 'normal' ? '00' : 'F0';
@@ -583,6 +698,13 @@ sub _send_sms_pdu {
 	# TODO Validity period (now fixed to 4 days)
 	my $vp = 'AA';
 
+	# Status report requested?
+	my $status_report = 0;
+	if( exists $opt{'status_report'} && $opt{'status_report'} )
+	{
+		$status_report = 1;
+	}
+
 	my $lOk = 0;
 	my $cReply;
 
@@ -607,7 +729,17 @@ sub _send_sms_pdu {
 	$me->log->write('info', 'encoded 7bit text (w/length) is ['.$enc_msg.']');
 
 	# Build PDU data
-	my $pdu = uc join( '', '00', '11', '00', $enc_da, '00', $class, $vp, $enc_msg );
+	my $pdu = uc join(
+		'',
+		'00',
+		($status_report ? '31' : '11'),
+		'00',
+		$enc_da,
+		'00',
+		$class,
+		$vp,
+		$enc_msg
+	);
 
 	$me->log->write('info', 'due to send PDU ['.$pdu.']');
 
@@ -617,34 +749,37 @@ sub _send_sms_pdu {
 
 	# Select PDU format for messages
 	$me->atsend(  q[AT+CMGF=0] . Device::Modem::CR );
+	$me->answer($Device::Modem::STD_RESPONSE);
 	$me->log->write('info', 'Selected PDU format for msg sending');
 
 	# Send SMS length
 	$me->atsend( qq[AT+CMGS=$len] . Device::Modem::CR );
+	$me->answer($Device::Modem::STD_RESPONSE);
 
 	# Sending SMS content encoded as PDU
 	$me->log->write('info', 'PDU sent ['.$pdu.' + CTRLZ]' );
 	$me->atsend( $pdu . Device::Modem::CTRL_Z );
-	$me->wait(2000);
 
 	# Get reply and check for errors
-	$cReply = $me->answer();
+	$cReply = $me->answer($Device::Modem::STD_RESPONSE, 30000);
+	$me->log->write( 'debug', "SMS reply: $cReply\r\n" );
 
-	if( $cReply =~ /ERROR/i ) {
-		$me->log->write( 'warning', "ERROR in sending SMS" );
-	} else {
+	if( $cReply =~ /OK$/i ) {
 		$me->log->write( 'info', "Sent SMS (pdu mode) to $num!" );
 		$lOk = 1;
+	} else {
+		$cReply =~ /(\+CMGS:.*)/;
+		$me->log->write( 'warning', "ERROR in sending SMS: $1" );
 	}
 
-	$lOk
+	return $lOk;
 }
 
 
 #
 # Set or request service center number
 #
-sub service_center(;$) {
+sub service_center {
 
 	my $self = shift;
 	my $nCenter;
@@ -662,7 +797,7 @@ sub service_center(;$) {
 		$self->atsend( qq[AT+CSCA="$nCenter"] . Device::Modem::CR );
 
 		# Check for modem answer
-		$lOk = ( $self->answer =~ /OK/ );
+		$lOk = ( $self->answer($Device::Modem::STD_RESPONSE) =~ /OK/ );
 
 		if( $lOk ) {
 			$self->log->write('info', 'service center number ['.$nCenter.'] stored');
@@ -676,7 +811,7 @@ sub service_center(;$) {
 		$self->atsend('AT+CSCA?' . Device::Modem::CR );
 
 		# Get answer and check for errors
-		($code, $nCenter) = $self->parse_answer();
+		($code, $nCenter) = $self->parse_answer($Device::Modem::STD_RESPONSE);
 
 		if( $code =~ /ERROR/ ) {
 			$self->log->write('warning', 'error status for "service_center" command');
@@ -696,22 +831,52 @@ sub service_center(;$) {
 
 }
 
+sub network
+{
+    my $self = $_[0];
+    my $network;
 
-2703;
+    #if( ! $self->test_command('COPS') )
+    #{
+    #    print 'NO COMMAND';
+    #    return undef;
+    #}
 
+    $self->atsend('AT+COPS?' . Device::Modem::CR);
 
+    # Parse COPS answer, the 3rd string is the network name
+    my $ans = $self->answer();
+    if( $ans =~ /"([^"]*)"/ )
+    {
+        $network = $1;
+        $self->log->write('info', 'Received network name ['.$network.']');
+    }
+    else
+    {
+        $self->log->write('info', 'Received no network name');
+    }
+
+    # Try to decode the network name
+    require Device::Gsm::Networks;
+    my $netname = Device::Gsm::Networks::name($network);
+    if( ! defined $netname || $netname eq 'unknown' )
+    {
+        $netname = undef;
+    }
+
+    return wantarray
+        ? ($netname, $network)
+        : $netname;
+
+}
+
+1;
 
 __END__
 
 =head1 NAME
 
-Device::Gsm - Perl extension to interface GSM cellular / modems
-
-=head1 WARNING
-
-   This is C<PRE-ALPHA> software, still needs extensive testing and
-   support for custom GSM commands, so use it at your own risk,
-   and without C<ANY> warranty! Have fun.
+Device::Gsm - Perl extension to interface GSM phones / modems
 
 =head1 SYNOPSIS
 
@@ -746,7 +911,7 @@ This class supports also C<PDU> mode to send C<SMS> messages, and should be
 fairly usable. In the past, I have developed and tested it under Linux RedHat 7.1
 with a 16550 serial port and Siemens C35i/C45 GSM phones attached with
 a Siemens-compatible serial cable. Currently, I'm developing and testing this stuff
-with Linux Slackware 9.1 and a B<Cambridge Silicon Radio> (CSR) USB
+with Linux Slackware 10.2 and a B<Cambridge Silicon Radio> (CSR) USB
 bluetooth dongle, connecting to a Nokia 6600 phone.
 
 Please be kind to the universe and contact me if you have troubles or you are
@@ -823,6 +988,31 @@ Example:
 
     $gsm->delete_sms(3);
 
+An optional second parameter specifies the "storage". It allows to delete messages
+from gsm phone memory or sim card memory. Example:
+
+    # Deletes first message from gsm phone memory
+    $gsm->delete_sms(1, 'ME');
+
+    # Deletes 3rd message from sim card
+    $gsm->delete_sms(3, 'SM');
+
+By default, it uses the currently set storage, via the C<storage()> method.
+
+=head2 forward()
+
+Sets call forwarding. Accepts three arguments: reason, mode and number.
+Reason can be the string C<unconditional>, C<busy>, C<no reply> and C<unreachable>.
+Mode can be the string C<disable>, C<enable>, C<query>, C<register>, C<erase>.
+
+Example:
+
+    # Set unconditional call forwarding to +47 123456789
+    $gsm->forward('unconditional','register','+47123456789');
+
+    # Erase unconditional call forwarding
+    $gsm->forward('unconditional','erase');
+
 
 =head2 hangup()
 
@@ -859,10 +1049,39 @@ C<Siemens>, C<Falcom>, ...). Example:
 =head2 messages()
 
 This method is a somewhat unstable and subject to change, but for now it seems to work.
-It is meant to extract all text SMS messages stored on your SIM card.
+It is meant to extract all text SMS messages stored on your SIM card or gsm phone.
 In list context, it returns a list of messages (or undefined value if no message or errors),
 every message being a C<Device::Gsm::Sms> object.
 
+The only parameter specifies the C<storage> where you want to read the messages,
+and can assume some of the following values (but check your phone/modem manual for
+special manufacturer values):
+
+=over 4
+
+=item C<ME>
+
+Means gsm phone B<ME>mory
+
+=item C<MT>
+
+Means gsm phone B<ME>mory on Nokia phones?
+
+=item C<SM>
+
+Means B<S>im card B<M>emory (default value)
+
+=back
+
+Example:
+
+    my $gsm = Device::Gsm->new();
+    $gsm->connect(port=>'/dev/ttyS0') or die "Can't connect!";
+
+    for( $gsm->messages('SM') )
+    {
+        print $_->sender(), ': ', $_->content(), "\n";
+    }
 
 =head2 mode()
 
@@ -886,6 +1105,25 @@ For example, for Siemens C45, C<$model> holds C<C45>; for Nokia 6600, C<$model>
 holds C<6600>.
 
 
+=head2 network()
+
+Returns the current registered or preferred GSM network operator. Example:
+
+	my $net_name = $gsm->network();
+    # Returns 'Wind Telecom Spa'
+
+    my($net_name, $net_code) = $gsm->network();
+    # Returns ('Wind Telecom Spa', '222 88')
+
+This obviously varies depending on country and network operator. For me now,
+it holds "Wind Telecomunicazioni SpA". It is not guaranteed that the mobile
+phone returns the decoded network name. It can also return a gsm network code,
+like C<222 88>. In this case, an attempt to decode the network name is made.
+
+Be sure to call the C<network()> method when already registered to gsm
+network. See C<register()> method.
+
+
 =head2 signal_quality()
 
 Returns the measure of signal quality expressed in dBm units, where near to zero is better.
@@ -905,6 +1143,24 @@ Returns the device firmare version, as stored by the manufacturer. Example:
 
 For example, for my Siemens C45, C<$rev> holds C<06>.
 
+=head2 storage()
+
+Allows to get/set the current sms storage, that is where the sms messages are saved,
+either the sim card or gsm phone memory. Phones/modems that do not support this feature
+(implemented by C<+CPMS> AT command won't be affected by this method.
+
+    my @msg;
+	my $storage = $gsm->storage();
+    print "Current storage is $storage\n";
+
+    # Read all messages on sim card
+    $gsm->storage('SM');
+    @msg = $gsm->messages();
+
+    # Read messages from gsm phone memory
+    $gsm->storage('ME');
+    push @msg, $gsm->messages();
+
 =head2 test_command()
 
 This method allows to query the device to know if a specific AT GSM command is supported.
@@ -993,6 +1249,17 @@ It is just a matter of trying.
 
 Phone number of message recipient
 
+=item C<status_report>
+
+If present with a true value, it enables sending of SMS messages (only for PDU mode,
+text mode SMS won't be influenced by this parameter) with the status report,
+also known as delivery report, that is a short message that reports the status
+of your sent message.
+Usually this is only available if your mobile company supports this feature,
+and probably you will be charged a small amount for this service.
+
+More information on this would be welcome.
+
 =back
 
 =head2 service_center()
